#include <J3ML/J3ML.hpp>
#include <Engine/types/entity/Entity.h>
#include <Engine/Logger.h>

using namespace Engine;

void Entity::AppendChild(Entity* entity) {
    if (entity->parent)
        entity->parent->RemoveChild(entity);

    entity->parent = this;
    children.push_back(entity);
}

Entity::~Entity() {
    for (auto* e : children)
        delete e;
}

void Entity::UpdateChildren() {
    for (auto& e : children) {
        if (!e)
            continue;

        e->Update();

        if (!e->children.empty())
            e->UpdateChildren();
    }
}

void Entity::DestroyChild(Entity* entity) {
    auto it = std::find(children.begin(), children.end(), entity);
    if (it != children.end())
        delete *it, children.erase(it);
}

void Entity::RemoveChild(Entity* entity) {
    auto it = std::find(children.begin(), children.end(), entity);
    if (it != children.end())
        children.erase(it);
    entity->parent = nullptr;
}

long Entity::GetCreationTime() const {
    return creation_time;
}

long Entity::GetAge() const {
    return std::chrono::duration_cast<std::chrono::microseconds>(std::chrono::high_resolution_clock::now().time_since_epoch()).count() - creation_time;
}

Entity* Entity::GetParent() const {
    return parent;
}

std::vector<Entity*> Entity::GetChildren() const {
    return children;
}

std::vector<Entity*> Entity::GetAllDescendents(const Entity* entity) {
    std::vector<Entity*> result = entity->GetChildren();

    if (result.empty())
        return result;

    for (auto* c : entity->GetChildren()) {
        auto descendants = Entity::GetAllDescendents(c);
        result.insert(result.end(), descendants.begin(), descendants.end());
    }
    return result;
}

Scene* Entity::GetScene() const {
    if (scene)
        return scene;

    const Entity* top = GetParent();
    while (top && top->GetParent())
        top = top->GetParent();

    if (top->scene == nullptr)
        Logger::Fatal("Getting the pointer to the current scene but we've lost it somehow?");

    return top->scene;
}

Scene* Entity::GetScene() {
    const Entity* entity = this;
    scene = entity->GetScene();
    return scene;
}
